import { Contract, utils } from 'ethers';
import { networkConfigs } from './utils/wallet-helper';
import { handleError } from './utils/utils';
import { _abi } from './contracts/ERC721Offsetable';
export var MintStatus;
(function (MintStatus) {
  MintStatus[MintStatus["CLOSED"] = 0] = "CLOSED";
  MintStatus[MintStatus["PUBLIC"] = 1] = "PUBLIC";
  MintStatus[MintStatus["PRIVATE"] = 2] = "PRIVATE";
})(MintStatus || (MintStatus = {}));
export class Block {
  constructor(contractAddress, testnet, type, walletInfo) {
    this.contractAddress = contractAddress;
    this.testnet = testnet;
    this.size = 0;
    this.contract = {};
    this.walletInfo = walletInfo;
    this.mintPrice = 0;
    this.type = type;
    this.status = MintStatus.CLOSED;
    this.personalAllocationLeft = 0;
    this.tokensOwned = 0;
  }
  async init() {
    try {
      if (this.type === 'ERC721') {
        if (!this.contractAddress.match(/^0x[a-fA-F0-9]{40}$/)) {
          throw new Error('Missing or invalid contract address');
        }
        await this._checkChain();
        await this._getContractDetails();
        window.ethereum.on('disconnect', () => {
          console.log('Disconnected');
          window.location.reload();
        });
        window.ethereum.on('accountsChanged', () => {
          console.log('tactical reload');
          window.location.reload();
        });
      }
    }
    catch (error) {
      console.log(error);
    }
  }
  static async create(contractAddress, testnet, type, walletInfo) {
    try {
      const block = new Block(contractAddress, testnet, type, walletInfo);
      await block.init();
      return block;
    }
    catch (error) {
      return null;
    }
  }
  async publicMint(amount) {
    const estimate = await this.contract.estimateGas.publicMint(amount);
    const safeGasLimit = Math.floor(parseInt(estimate.toString()) * 1.2);
    const tx = await this.contract.publicMint(amount, { gasLimit: safeGasLimit });
    console.log(this._getTxReceipt(tx.hash));
  }
  async _getContractDetails() {
    try {
      this.contract = new Contract(this.contractAddress, _abi, this.walletInfo.signer);
      if (!this.contract) {
        throw new Error('Contract initialization failed.');
      }
      const [, mintPrice, maxPerAddress, status] = await this.contract.mintConfig();
      this.totalSupply = (await this.contract.totalSupply()).toNumber();
      this.maxSupply = (await this.contract.maxSupply()).toNumber();
      this.tokensOwned = (await this.contract.tokensOfOwner(this.walletInfo.signerAddress)).length;
      this.personalAllocationLeft = Math.max(0, maxPerAddress - this.tokensOwned);
      console.log(this.personalAllocationLeft);
      this.mintPrice = parseFloat(utils.formatEther(mintPrice));
      this.maxPerAddress = maxPerAddress;
      this.status = status;
    }
    catch (error) {
      handleError(error);
    }
  }
  async _getTxReceipt(hash) {
    return await this.walletInfo.provider.getTransactionReceipt(hash);
  }
  async _checkChain() {
    const requiredNetwork = this.testnet ? networkConfigs.rinkeby : networkConfigs.homestead;
    if (!((await this.walletInfo.provider.getNetwork()).name === requiredNetwork.name)) {
      try {
        await window.ethereum.request({
          method: 'wallet_switchEthereumChain',
          params: [{ chainId: requiredNetwork.chainId }], // chainId must be in hexadecimal numbers
        });
        window.location.reload();
      }
      catch (error) {
        handleError(error);
      }
    }
  }
}
